# -*- perl -*-

# t/001_load.t - check module loading and create testing directory

use Test::More tests => 1;

BEGIN { use_ok( 'Data::Utilities' ); }

# my $object = Data::Utilities->new ();
# isa_ok ($object, 'Data::Utilities');


